// Runge Kutta method of the fourth order for solving an ODE

// Suppose we wish to solve the following ODE: 
//     y' = y
// with the initial point, 
//     y0 = 1
// The analytical solution for this is:
//     y = y0*e^(x - x0)

clc
clear

// This is our ODE:
function dy = f(y)
    dy = y
endfunction

// Over the following points: 
x = 0:0.5:10;
// We see the difference is: 
h = 0.5
// therefore, 
y(1) = 1 // since indexing begins at 1.

// We implement the Runge Kutta method: 
// The Runge Kutta method (fourth order) is 
//     y{n+1} = yn + (1/6)*h(k1 + 2*k2 + 2*k3 + k4)
// where,
//     k1 = f(xn          , yn)
//     k2 = f(xn + (1/2)*h, yn + (1/2)*h*k1)
//     k3 = f(xn + (1/2)*h, yn + (1/2)*h*k2)
//     k4 = f(xn + h      , yn + h*k3)

for i = 1:length(x),
    k1 = f(y(i));
    k2 = f(y(i) + 0.5*h*k1);
    k3 = f(y(i) + 0.5*h*k2);
    k4 = f(y(i) + h*k3);
    y(i+1) = y(i) + (1/6)*h*(k1 + 2*k2 + 2*k3 + k4);;
end

// We compare the answer we have obtained using the analytical solution: 
plot(exp(x'), 'green')
plot(y, 'black')